import tensorflow as tf
import context
from dnc import *

dtype=tf.float32
batch_size = 32
sequence_length = 5
sequence_padding = 1
sequence_width = 5
num_sequences = 10
total_sequence_length = (sequence_length + sequence_padding) * 2 * num_sequences

input_shape = [batch_size, total_sequence_length, sequence_width]
input_sequences = tf.placeholder(shape=input_shape, dtype=dtype)

controller_network = SimpleFeedforwardController(20, 2)

dnc = DifferentiableNeuralComputer(
  controller_network,
  memory_locations=10,
  memory_width=sequence_width,
  num_read_heads=2)

outputs, _ = tf.nn.dynamic_rnn(dnc, input_sequences, dtype=dtype)

weights = tf.Variable(tf.truncated_normal([dnc.output_size, sequence_width]))
bias = tf.Variable(tf.truncated_normal([sequence_width]))
output_sequences = tf.reshape(
  tf.matmul(tf.reshape(outputs, [-1, dnc.output_size]), weights) + bias,
  input_shape)

output = tf.maximum(tf.sign(output_sequences), 0)
# loss = tf.nn.sigmoid_cross_entropy_with_logits(output_sequences, required_output(input_sequences))


def random_sequence(dtype):
  sequence = tf.cast(tf.random_uniform(shape=[batch_size, sequence_length, sequence_width], minval=0, maxval=2, dtype=tf.int32), dtype)
  return tf.pad(sequence, paddings=[[0,0], [sequence_padding, sequence_length + sequence_padding], [0,0]])

def n_random_sequences(n, dtype):
  return tf.concat(1, [random_sequence(dtype) for _ in range(n)])

def required_output(input_sequences):
  return tf.pad(tf.slice(input_sequences, [0, 0, 0], [-1, total_sequence_length - (sequence_length + sequence_padding), -1]), [[0,0],[6,0],[0,0]])

with tf.Session():
  print(required_output(n_random_sequences(num_sequences, dtype)).eval())
